///////////////////////////////////////////////////////////////////////////////////////////////
//
// Description:
//
//		This code example outlines the approaches for calling MailEnable 
//		components from Microsoft Visual C++
//
//		There are two examples provided, an "Early Bound" example and a "Late Bound" example.
//
// Author: Andrew Sproul
//
// Revision History: (Not Controlled Document)
//
///////////////////////////////////////////////////////////////////////////////////////////////

//
// The following is only necessary for early bound example
//
#import "MEAOPO.dll" high_property_prefixes("GetProp_","PutProp_","PutRefProp_")
using namespace MEAOPO;

#include <windows.h>
#include <stdio.h>

//
// The following are only necessary for the late bound example
//

#include <comdef.h>
#include <atlbase.h>
CComModule _Module; //needed for late bound example
#include <atlcom.h>

//
// The following is a helper function to conver BSTRs to LPSZ
//
LPTSTR Convert(BSTR pStr,LPTSTR szStr,INT nSize)
{
   DWORD dwRet = WideCharToMultiByte(CP_ACP,0,(LPCWSTR)pStr,-1,szStr,nSize,NULL,NULL);
   return szStr;
}

//
// Early Binding Example using the CComDispatchDriver ATL Class
// 

int EarlyBoundExample()
{
	try
	{
		_PostofficePtr obj;
		obj = _PostofficePtr("MEAOPO.Postoffice");
		obj->PutProp_Name("MailEnable");
		obj->PutProp_Account("MailEnable");
		obj->PutProp_Status(1);
		if (obj->GetPostoffice() == 1)
		{
			printf("Postoffice exists.\n");
			return false;
		}
	}
	catch (_com_error &error)
	{
		printf("%s\n",error.ErrorMessage());
		return false;
	}
	return true;
}

//
// Late Binding Example using the CComDispatchDriver ATL Class
// 

int LateBoundExample()
{
	CComDispatchDriver  oPostoffice;
	CComPtr<IDispatch>  lpTDispatch;
	_variant_t vResult;
	_variant_t vProperty;
	//char lpResult[512];
	DWORD dwResult = 0;
	HRESULT hr = 0;
	char lpszReturnResult[512];
	char lpszError[256];
	char lpszErrorString[128];

	hr = lpTDispatch.CoCreateInstance(_bstr_t("MEAOPO.Postoffice"));
	if ( FAILED(hr) )
	{
		goto Exit;   // All errors need checking!
	}
	//
	// Assign the interface ptr to our Dispatch Driver
	//
	oPostoffice = lpTDispatch;  
	//
	// Set the properties of the object before calling our method
	//
	hr = oPostoffice.PutPropertyByName(_bstr_t("Name"),&_variant_t("MailEnable"));
	hr = oPostoffice.PutPropertyByName(_bstr_t("Account"),&_variant_t("MailEnable"));
	hr = oPostoffice.PutPropertyByName(_bstr_t("Status"),&_variant_t(-1L));
	//
	// Call the method of the Postoffice Object with no Parameters ie: (Invoke0)
	// Note: Need to use InvokeN if using the Edit function
	//
	hr = oPostoffice.Invoke0(_bstr_t("GetPostoffice"),&vResult);
	if ( FAILED(hr) )
	{
		goto Exit;
	}
	//
	// Check the resulting Values of the call starting with the return value itself
	//
	dwResult = vResult.lVal;
	printf("Function Returned Value = %ld\n",dwResult);
	//
	// Now get the actual properties (if we need them)
	//
	hr = oPostoffice.GetPropertyByName(_bstr_t("Account"),&vProperty);
	if ( FAILED(hr) )
	{
		goto Exit;
	}
	printf("Returned Postoffice Value = %s\n",Convert(vProperty.bstrVal,lpszReturnResult,sizeof(lpszReturnResult)));

Exit:
	if (FAILED(hr)) 
	{
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,0,hr, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),lpszErrorString,sizeof(lpszErrorString),0);
		sprintf(lpszError,"Our sample incurred an error: (%ld) %s",hr,lpszErrorString);
		printf("%s\n",lpszError);
		return FALSE;
	}
	return TRUE;
}

int main()
{
	//
	// Initialize COM
	//
	CoInitialize(NULL);
	//
	// Execute our early bound example
	//
	EarlyBoundExample();
	//
	// Execute our late bound example
	//
	LateBoundExample();
	//
	// Uninitialize COM
	//
	CoUninitialize();

	return 1;
}